/*
  ClientUDP.cpp - Ethernet UDP library for Wiring & Arduino,
  based on Client.cpp from Arduino IDE 0016 library.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  2009-Jun-25	v1.0, created by Ilya Danilov <arduino@idanilov.ru>
  2009-Jul-12   v1.1, added _sent and open retucn codes by Ilya Danilov <arduino@idanilov.ru>

  
*/

extern "C" {
  #include "types.h"
  #include "w5100.h"
  #include "socket.h"
}

#include "WProgram.h"
#include "Ethernet.h"
#include "ClientUDP.h"

uint16_t ClientUDP::_srcport = 0;

ClientUDP::ClientUDP(uint8_t sock) {
  _sock = sock;
  _sent = 0; 
}

ClientUDP::ClientUDP(uint8_t *ip, uint16_t port) {
  _ip = ip;
  _port = port;
  _sock = 255;  
  _sent = 0; 
}

uint8_t ClientUDP::open(uint16_t a_src_port) {
  if (_sock != 255) 
    return ERR_UDP_SOCKET_USED;
  
  for (int i = 0; i < MAX_SOCK_NUM; i++) {
    uint8_t s = getSn_SR(i);
    if (s == SOCK_CLOSED || s == SOCK_FIN_WAIT) {
      _sock = i;
      break;
    }
  }
  
  if (_sock == 255)
    return ERR_UDP_NO_SOCK_AVAIL;
    
  if (!a_src_port) { 
    _srcport++;
    if (_srcport+1024 == 0) _srcport = 0;
  }
    
  socket(_sock, Sn_MR_UDP, a_src_port ? a_src_port : (_srcport+1024), 0);
   
  delay(10);

  if (status() == SOCK_CLOSED) {
    _sock = 255;  
    return ERR_UDP_OPEN_FAILED;
  }
  
  return ERR_UDP_NO_ERROR;
}

void ClientUDP::write(uint8_t b) {
  if (_sock != 255) 
    _sent = sendto(_sock, &b, 1, _ip, _port);
}

void ClientUDP::write(const uint8_t *buf, size_t size) {
  if (_sock != 255) 
    _sent = sendto(_sock, buf, size, _ip, _port);
}

int ClientUDP::available() {
  if (_sock != 255)    
    return getSn_RX_RSR(_sock);  
  return 0;
}

uint16_t ClientUDP::read(uint8_t *ip, uint16_t *port, uint8_t *buf, uint16_t len) {
  if (_sock != 255) 
    return recvfrom(_sock,buf,len,ip,port);
  return 0;
}

void ClientUDP::stop() {
  if (_sock == 255)
    return;
  close(_sock);
  _sock = 255;
}

uint8_t ClientUDP::opened() {
  uint8_t s = status();
  return !(s == SOCK_CLOSED);
}

uint8_t ClientUDP::status() {
  return getSn_SR(_sock);
}
